import yfinance as yf
import pandas as pd

# Read ticker symbols from the input file
stockList = []
inputFileName = 'Nasdaq100.txt'
with open(inputFileName, "r") as f:
    for line in f:
        ticker = line.strip()
        if ticker:
            stockList.append(ticker)

# Define the start and end dates
start_date = "2021-01-01"
end_date = "2025-01-01"

for ticker in stockList:
    try:
        # Fetch historical data with auto_adjust set to False so that "Adj Close" is included
        data = yf.download(ticker, start=start_date, end=end_date, auto_adjust=False)

        if data.empty:
            print(f"No data returned for ticker {ticker}.")
            continue

        # Reset the index to bring the Date into a column
        data.reset_index(inplace=True)

        # Check if 'Adj Close' is present; if not, substitute with 'Close'
        if 'Adj Close' not in data.columns:
            print(f"Ticker {ticker} is missing 'Adj Close'. Using 'Close' as substitute.")
            data['Adj Close'] = data['Close']

        # Define required columns and check if any are missing
        required_columns = ['Date', 'Open', 'High', 'Low', 'Close', 'Adj Close', 'Volume']
        missing_cols = [col for col in required_columns if col not in data.columns]
        if missing_cols:
            print(f"Ticker {ticker} is missing columns: {missing_cols}. Skipping ticker.")
            continue

        # Reorder the columns to the desired output order
        data = data[required_columns]

        # Save the DataFrame to a CSV file without header and index, formatting floats to 6 decimal places
        output_filename = f"{ticker}.csv"
        data.to_csv(output_filename, index=False, header=False, float_format="%.6f")
        print(f"Saved data for {ticker} to {output_filename}")

    except Exception as e:
        print(f"Failed to get ticker '{ticker}' reason: {e}")

print("Data fetching complete.")
